#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.client.TalosClientConfigkeys import TalosClientConfigKeys


class TalosProducerConfigKeys(TalosClientConfigKeys):
    # The producer max retry times when putMessage failed, -1 represent don't retry
    GALAXY_TALOS_PRODUCER_MAX_RETRY_TIMES = "galaxy.talos.producer.max.retry.times"
    GALAXY_TALOS_PRODUCER_MAX_RETRY_TIMES_DEFAULT = 3

    # The producer partitionSender sleep / delay time when continuous failure
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_BASE_BACKOFF_TIME = "galaxy.talos.producer.put.message.base.backoff.time.milli"
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_BASE_BACKOFF_TIME_DEFAULT = 100

    # The producer partitionSender max sleep / delay time when continuous failure
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_MAX_BACKOFF_TIME = "galaxy.talos.producer.put.message.max.backoff.time.milli"
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_MAX_BACKOFF_TIME_DEFAULT = 300

    # The producer partitionSender putMessage max failed times
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_MAX_FAILED_TIMES = "galaxy.talos.producer.put.message.max.failed.times"
    GALAXY_TALOS_PRODUCER_PUT_MESSAGE_MAX_FAILED_TIMES_DEFAULT = 10

    # The producer buffered message number for one partition
    # if the total number exceed it, not allowed to addMessage for user
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_NUMBER = "galaxy.talos.producer.max.buffered.message.number"
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_NUMBER_DEFAULT = 1000000

    # The producer buffered message bytes for one partition
    # if the total bytes exceed it, not allowed to addMessage for user
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_BYTES = "galaxy.talos.producer.max.buffered.message.bytes"
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_BYTES_DEFAULT = 500 * 1024 * 1024

    # The producer buffered message time for one partition
    # if the oldest buffered message always exceed the time,
    # putMessage will always be called
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS = "galaxy.talos.producer.max.buffered.milli.secs"
    GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS_DEFAULT = 200

    # The producer max number of message in each putMessage batch
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER = "galaxy.talos.producer.max.put.message.number"
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_DEFAULT = 2000
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_MINIMUM = 1
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_MAXIMUM = 5000

    # The producer max bytes of message in each putMessage batch
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES = "galaxy.talos.producer.max.put.message.bytes"
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_DEFAULT = 1024 * 1024
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_MINIMUM = 1
    GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_MAXIMUM = 10 * 1024 * 1024

    # The producer thread pool number
    GALAXY_TALOS_PRODUCER_THREAD_POOL_SIZE = "galaxy.talos.producer.thread.pool.size"
    GALAXY_TALOS_PRODUCER_THREAD_POOL_SIZE_DEFAULT = 16

    # The producer scan / update partition number interval
    GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL = "galaxy.talos.producer.check.partition.interval"
    GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_DEFAULT = 180
    GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_MINIMUM = 60
    GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_MAXIMUM = 300

    # The producer update partitionId time interval when calling addMessage
    # 100 milli secs by default
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL = "galaxy.talos.producer.update.partition.id.interval.milli"
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_DEFAULT = 0.1
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_MINIMUM = 0.001
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_MAXIMUM = 0.5

    # The producer update partitionId
    # when message number added one partition enough large
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITION_MSGNUMBER = "galaxy.talos.producer.update.partition.msgnumber"
    GALAXY_TALOS_PRODUCER_UPDATE_PARTITION_MSGNUMBER_DEFAULT = 1000

    # The producer partitionSender sleep / delay time when partitionNotServing
    GALAXY_TALOS_PRODUCER_WAIT_PARTITION_WORKING_TIME = "galaxy.talos.producer.wait.partition.working.time.milli"
    GALAXY_TALOS_PRODUCER_WAIT_PARTITION_WORKING_TIME_DEFAULT = 0.2

    # The producer compression type, right now suport "NONE", "SNAPPY" and "GZIP";
    # default is "SNAPPY";
    GALAXY_TALOS_PRODUCER_COMPRESSION_TYPE = "galaxy.talos.producer.compression.type"
    GALAXY_TALOS_PRODUCER_COMPRESSION_TYPE_DEFAULT = "SNAPPY"


#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

import unittest

from talos.thrift.message.ttypes import MessageAndOffset
from talos.thrift.message.ttypes import Message
from talos.thrift.consumer.ttypes import UpdateOffsetResponse
from talos.thrift.consumer.ttypes import LockPartitionResponse
from talos.thrift.consumer.ttypes import QueryOffsetResponse
from talos.thrift.consumer.ttypes import QueryOffsetRequest
from talos.thrift.consumer.ttypes import UpdateOffsetRequest
from talos.thrift.consumer.ttypes import CheckPoint
from talos.thrift.topic.ttypes import TopicTalosResourceName
from talos.thrift.topic.ttypes import TopicAndPartition
from talos.thrift.common.ttypes import GalaxyTalosException
from talos.thrift.common.ttypes import ErrorCode
from talos.consumer.SimpleConsumer import SimpleConsumer
from talos.consumer.MessageReader import MessageReader
from talos.consumer.TalosMessageReader import TalosMessageReader
from talos.consumer.PartitionFetcher import PartitionFetcher
from talos.consumer.MessageProcessor import MessageProcessor
from talos.client.TalosClientFactory import ConsumerClient
from talos.client.TalosClientFactory import MessageClient
from talos.client.TalosClientFactory import TopicClient
from talos.client.TalosClientConfig import TalosClientConfig
from talos.client.TalosClientConfigkeys import TalosClientConfigKeys
from mock import Mock
import time


class test_SimpleConsumer(unittest.TestCase):
    topicName = "MyTopic"
    resourceName = "12345#MyTopic#34595fkdiso456i390"
    talosResourceName = TopicTalosResourceName(resourceName)
    partitionId = 7
    partitionNum = 10
    startOffset = 0
    topicAndPartition = TopicAndPartition
    producerConfig = TalosClientConfig
    consumerConfig = TalosClientConfig
    messageClientMock = MessageClient
    topicClientMock = TopicClient
    messageList = []
    messageAndOffsetList = []
    simpleConsumer = SimpleConsumer



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 


class Constants:
    # Constants for rest sever path
    TALOS_API_ROOT_PATH = "/v1/api"
    TALOS_TOPIC_SERVICE_PATH = TALOS_API_ROOT_PATH + "/topic"
    TALOS_MESSAGE_SERVICE_PATH = TALOS_API_ROOT_PATH + "/message"
    TALOS_QUOTA_SERVICE_PATH = TALOS_API_ROOT_PATH + "/quota"
    TALOS_CONSUMER_SERVICE_PATH = TALOS_API_ROOT_PATH + "/consumer"
    TALOS_METRIC_SERVICE_PATH = TALOS_API_ROOT_PATH + "/metric"

    TALOS_IDENTIFIER_DELIMITER = "#"
    TALOS_CONNECTION_DELIMITER = "-"
    TALOS_NAME_REGEX = "^(?!_)(?!-)(?!.*?_$)[a-zA-Z0-9_-]+$"
    TALOS_TEMPORARY_FILE_SUFFIX = ".tmp"

    # Constants for producer
    TALOS_SINGLE_MESSAGE_BYTES_MINIMAL = 1
    TALOS_SINGLE_MESSAGE_BYTES_MAXIMAL = 10 * 1024 * 1024
    TALOS_MESSAGE_BLOCK_BYTES_MAXIMAL = 20 * 1024 * 1024

    TALOS_PARTITION_KEY_LENGTH_MINIMAL = 1
    TALOS_PARTITION_KEY_LENGTH_MAXIMAL = 256

    # Constants for cloud - manager auth
    TALOS_CLOUD_TOPIC_NAME_DELIMITER = "/"
    TALOS_CLOUD_ORG_PREFIX = "CL"
    TALOS_CLOUD_TEAM_PREFIX = "CI"
    TALOS_CLOUD_AK_PREFIX = "AK"
    TALOS_GALAXY_AK_PREFIX = "EAK"



import unittest

from talos.thrift.message.ttypes import Message
from talos.client.TalosClientConfig import TalosClientConfig
from talos.client.TalosClientConfigkeys import TalosClientConfigKeys
from talos.producer.TalosProducerConfigKey import TalosProducerConfigKeys
from talos.producer.TalosProducer import TalosProducer
from talos.producer.PartitionMessageQueue import PartitionMessageQueue
from talos.producer.UserMessage import UserMessage
from mock import Mock


class test_PartitionMessageQueue(unittest.TestCase):
	msgStr = "hello, partitionMessageQueueTest"
	msg = Message(message=msgStr)

	partitionId = 7
	maxPutMsgNumber = 3
	maxBufferedMillSecs = 200

	producerConfig = TalosClientConfig
	partitionMessageQueue = PartitionMessageQueue
	userMessage1 = UserMessage
	userMessage2 = UserMessage
	userMessage3 = UserMessage
	userMessageList = []
	messageList = []

	producer = Mock(TalosProducer)

	def setUp(self):
		properties = dict()
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER] \
			= self.maxPutMsgNumber
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS] \
			= self.maxBufferedMillSecs
		properties[TalosClientConfigKeys.GALAXY_TALOS_SERVICE_ENDPOINT] = "testURI"
		self.producerConfig = TalosClientConfig(properties)

		self.userMessage1 = UserMessage(self.msg)
		self.userMessage2 = UserMessage(self.msg)
		self.userMessage3 = UserMessage(self.msg)

		self.partitionMessageQueue = PartitionMessageQueue(self.producerConfig, self.partitionId, self.producer)

	def tearDown(self):
		pass

	def test_add_get_message_when_max_put_number(self):
		self.userMessageList = [self.userMessage1, self.userMessage2]
		self.messageList = [self.msg, self.msg]

		self.producer.increase_buffered_count = Mock(return_value=None)
		self.producer.is_active = Mock(return_value=True)
		self.producer.decrease_buffered_count = Mock(return_value=None)

		self.partitionMessageQueue.add_message(self.userMessageList)
		# check log has waiting time
		self.assertEqual(len(self.messageList), len(self.partitionMessageQueue.get_message_list()))

	def test_add_get_message_when_not_alive(self):
		userMessageList = [self.userMessage1, self.userMessage2, self.userMessage3]
		messageList = [self.msg, self.msg, self.msg]

		self.producer.increase_buffered_count = Mock(return_value=None)
		self.producer.is_active = Mock(side_effect=[True, False])
		self.producer.decrease_buffered_count = Mock(return_value=None)

		self.partitionMessageQueue.add_message(userMessageList)
		# check log has waiting time
		self.assertEqual(len(messageList), len(self.partitionMessageQueue.get_message_list()))
		self.assertEquals(0, len(self.partitionMessageQueue.get_message_list()))

	def test_add_get_wait_message_when_not_alive(self):
		userMessageList = [self.userMessage1, self.userMessage2]
		messageList = [self.msg, self.msg]

		self.producer.increase_buffered_count = Mock(return_value=None)
		self.producer.is_active = Mock(side_effect=[True, True, False])
		self.producer.decrease_buffered_count = Mock(return_value=None)

		self.partitionMessageQueue.add_message(userMessageList)
		# check log has waiting	time
		self.assertEqual(len(messageList), len(self.partitionMessageQueue.get_message_list()))
		self.assertEquals(0, len(self.partitionMessageQueue.get_message_list()))


if __name__ == '__main__':
	unittest.main()



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.client.TalosClientConfig import TalosClientConfig
from talos.client.TalosClientConfig import TalosClientConfigKeys
from talos.thrift.auth.ttypes import Credential
from talos.thrift.topic.ttypes import TopicAndPartition
from talos.utils import Utils
from talos.client.TalosClientFactory import TalosClientFactory
from talos.client.TalosClientFactory import MessageClient
from talos.thrift.topic.TopicService import GetDescribeInfoRequest
from talos.thrift.message.MessageService import GetMessageRequest
from talos.thrift.message.MessageService import GetMessageResponse
from atomic import AtomicLong
from talos.client.compression.Compression import Compression
from talos.thrift.common.ttypes import GalaxyTalosException
from talos.thrift.message.ttypes import MessageOffset
import logging


class SimpleConsumer:
    logger = logging.getLogger("SimpleConsumer")
    consumerConfig = TalosClientConfig
    topicName = str
    partitionId = int
    credential = Credential
    topicAndPartition = TopicAndPartition
    messageClient = MessageClient
    simpleConsumerId = str
    requestId = AtomicLong

    def __init__(self, clientConfig=None, topicName=None, partitionId=None,
                 topicAndPartition=None, credential=None, messageClient=None):
        self.consumerConfig = clientConfig
        if topicName:
            Utils.check_topic_name(topicName)
            self.topicName = topicName
            self.partitionId = partitionId
            self.credential = credential
            talosClientFactory = TalosClientFactory(clientConfig, credential)
            self.messageClient = talosClientFactory.new_message_client()
            self.get_topic_info(talosClientFactory.new_topic_client(), topicName,
                                partitionId)
        else:
            self.messageClient = messageClient
            self.topicAndPartition = topicAndPartition
        self.simpleConsumerId = Utils.generate_client_id(clientConfig.get_client_ip(), "")
        self.requestId = AtomicLong(1)

    def get_topic_info(self, topicClient=None, topicName=None, partitionId=None):
        response = topicClient.get_describe_info(GetDescribeInfoRequest(topicName))
        self.topicAndPartition = TopicAndPartition(topicName,
                                                   response.topicTalosResourceName,
                                                   partitionId)

    def get_topic_talos_resource_name(self):
        return self.topicAndPartition.topicTalosResourceName

    def set_simple_consumer_id(self, simpleConsumerId=None):
        self.simpleConsumerId = simpleConsumerId

    def fetch_message(self, startOffset=None, maxFetchedNumber=None):
        Utils.check_start_offset_validity(startOffset)
        Utils.check_parameter_range(TalosClientConfigKeys.GALAXY_TALOS_CONSUMER_MAX_FETCH_RECORDS,
                                    maxFetchedNumber,
                                    TalosClientConfigKeys.GALAXY_TALOS_CONSUMER_MAX_FETCH_RECORDS_MINIMUM,
                                    TalosClientConfigKeys.GALAXY_TALOS_CONSUMER_MAX_FETCH_RECORDS_MAXIMUM)
        requestSequenceId = Utils.generate_request_sequence_id(self.simpleConsumerId,
                                                               self.requestId)

        # limit the default max fetch bytes 2M
        getMessageRequest = GetMessageRequest(self.topicAndPartition, startOffset)
        getMessageRequest.sequenceId = requestSequenceId
        getMessageRequest.maxGetMessageNumber = maxFetchedNumber
        getMessageRequest.maxGetMessageBytes = self.consumerConfig.get_max_fetch_msg_bytes()
        clientTimeout = self.consumerConfig.get_client_timeout()
        getMessageRequest.timeoutTimestamp = (Utils.current_time_mills() + clientTimeout)

        getMessageResponse = GetMessageResponse(messageBlocks=[])
        try:
            getMessageResponse = self.messageClient.get_message(getMessageRequest)
        except GalaxyTalosException as e:
            self.logger.error("fetch message failed! " + e.details)
            raise e

        messageAndOffsetList = Compression().decompress(getMessageResponse.messageBlocks,
                                                        getMessageResponse.unHandledMessageNumber)

        if len(messageAndOffsetList) <= 0:
            return messageAndOffsetList

        actualStartOffset = messageAndOffsetList[0].messageOffset

        if messageAndOffsetList[0].messageOffset == startOffset or startOffset == \
                MessageOffset.START_OFFSET or startOffset == MessageOffset.LATEST_OFFSET:
            return messageAndOffsetList
        else:
            start = int(startOffset - actualStartOffset)
            assert start > 0
            end = len(messageAndOffsetList)
            return messageAndOffsetList[start:end]






#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.client.serialization.MessageSerializer import MessageSerializer
from talos.client.serialization.MessageSerializerV1 import MessageSerializerV1
from talos.client.serialization.MessageSerializerV2 import MessageSerializerV2
from talos.client.serialization.MessageSerializerV3 import MessageSerializerV3
from io import BytesIO
import logging


class MessageSerialization(object):

    logger = logging.getLogger("MessageSerialization")

    def serialize_message(self, message=None, dataOutputIo=None, messageVersion=None):
        if messageVersion == 'V1':
            MessageSerializerV1().serialize(message, dataOutputIo)
        elif messageVersion == 'V2':
            MessageSerializerV2().serialize(message, dataOutputIo)
        elif messageVersion == 'V3':
            MessageSerializerV3().serialize(message, dataOutputIo)
        return

    def deserialize_message(self, dataInputIo=None):
        messageSerializer = MessageSerializer
        header = bytearray(messageSerializer.VERSION_NUMBER_LENGTH)
        try:
            if dataInputIo.readinto(header) != messageSerializer.VERSION_NUMBER_LENGTH:
                self.logger.error("read header error!")
                return
        except Exception as e:
            self.logger.error("read header error!" + e.message)
            raise
        messageVersion = MessageSerializer.decode_message_version(header=header)
        if messageVersion.get_version() == "V1":
            return MessageSerializerV1().deserialize(header, dataInputIo)
        elif messageVersion.get_version() == "V2":
            return MessageSerializerV2().deserialize(header, dataInputIo)
        elif messageVersion.get_version() == "V3":
            return MessageSerializerV3().deserialize(header, dataInputIo)

    def get_message_size(self, message=None, messageVersion=None):
        if messageVersion == 'V1':
            return MessageSerializerV1().get_message_size(message)
        elif messageVersion == 'V2':
            return MessageSerializerV2.get_message_size(message)
        elif messageVersion == 'V3':
            return MessageSerializerV3().get_message_size(message)






#coding=utf-8
#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from talos.thrift.Thrift import TType, TMessageType, TException, TApplicationException

from talos.thrift.transport import TTransport
from talos.thrift.protocol import TBinaryProtocol, TProtocol
try:
  from talos.thrift.protocol import fastbinary
except:
  fastbinary = None


class GrantType:
  DEVELOPER = 1
  APP_ROOT = 2
  APP_USER = 3
  GUEST = 4

  _VALUES_TO_NAMES = {
    1: "DEVELOPER",
    2: "APP_ROOT",
    3: "APP_USER",
    4: "GUEST",
  }

  _NAMES_TO_VALUES = {
    "DEVELOPER": 1,
    "APP_ROOT": 2,
    "APP_USER": 3,
    "GUEST": 4,
  }


class Grantee:
  """
  Attributes:
   - type: 被授权者类型
   - identifier: 被授权者唯一标识，若 GrantType 为 DEV 即 developerId， 其它则为appId
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'type', None, None, ), # 1
    (2, TType.STRING, 'identifier', None, None, ), # 2
  )

  def __init__(self, type=None, identifier=None,):
    self.type = type
    self.identifier = identifier

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.type = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.identifier = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Grantee')
    if self.type is not None:
      oprot.writeFieldBegin('type', TType.I32, 1)
      oprot.writeI32(self.type)
      oprot.writeFieldEnd()
    if self.identifier is not None:
      oprot.writeFieldBegin('identifier', TType.STRING, 2)
      oprot.writeString(self.identifier)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.type)
    value = (value * 31) ^ hash(self.identifier)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from talos.thrift.Thrift import TType, TMessageType, TException, TApplicationException
import talos.thrift.common.ttypes
import talos.thrift.quota.ttypes
import talos.thrift.authorization.ttypes


from talos.thrift.transport import TTransport
from talos.thrift.protocol import TBinaryProtocol, TProtocol
try:
  from talos.thrift.protocol import fastbinary
except:
  fastbinary = None


class TopicStatus:
  CREATING = 0
  ACTIVE = 1
  DELETING = 2
  DELETED = 3

  _VALUES_TO_NAMES = {
    0: "CREATING",
    1: "ACTIVE",
    2: "DELETING",
    3: "DELETED",
  }

  _NAMES_TO_VALUES = {
    "CREATING": 0,
    "ACTIVE": 1,
    "DELETING": 2,
    "DELETED": 3,
  }

class Permission:
  NONE = 0
  PUT_MESSAGE = 1
  GET_MESSAGE = 2
  FULL_MESSAGE_CONTROL = 3
  DESCRIBE_TOPIC = 4
  PUT_MESSAGE_AND_DESCRIBE_TOPIC = 5
  GET_MESSAGE_AND_DESCRIBE_TOPIC = 6
  TOPIC_READ_AND_MESSAGE_FULL_CONTROL = 7
  CHANGE_TOPIC = 8
  FULL_TOPIC_CONTROL = 12
  FULL_CONTROL = 15
  CHANGE_PERMISSION = 16
  ADMIN = 31

  _VALUES_TO_NAMES = {
    0: "NONE",
    1: "PUT_MESSAGE",
    2: "GET_MESSAGE",
    3: "FULL_MESSAGE_CONTROL",
    4: "DESCRIBE_TOPIC",
    5: "PUT_MESSAGE_AND_DESCRIBE_TOPIC",
    6: "GET_MESSAGE_AND_DESCRIBE_TOPIC",
    7: "TOPIC_READ_AND_MESSAGE_FULL_CONTROL",
    8: "CHANGE_TOPIC",
    12: "FULL_TOPIC_CONTROL",
    15: "FULL_CONTROL",
    16: "CHANGE_PERMISSION",
    31: "ADMIN",
  }

  _NAMES_TO_VALUES = {
    "NONE": 0,
    "PUT_MESSAGE": 1,
    "GET_MESSAGE": 2,
    "FULL_MESSAGE_CONTROL": 3,
    "DESCRIBE_TOPIC": 4,
    "PUT_MESSAGE_AND_DESCRIBE_TOPIC": 5,
    "GET_MESSAGE_AND_DESCRIBE_TOPIC": 6,
    "TOPIC_READ_AND_MESSAGE_FULL_CONTROL": 7,
    "CHANGE_TOPIC": 8,
    "FULL_TOPIC_CONTROL": 12,
    "FULL_CONTROL": 15,
    "CHANGE_PERMISSION": 16,
    "ADMIN": 31,
  }


class TopicTalosResourceName:
  """
  Attributes:
   - topicTalosResourceName: The Talos Resource Name for the topic;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicTalosResourceName', None, None, ), # 1
  )

  def __init__(self, topicTalosResourceName=None,):
    self.topicTalosResourceName = topicTalosResourceName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicTalosResourceName = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TopicTalosResourceName')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRING, 1)
      oprot.writeString(self.topicTalosResourceName)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class TopicAndPartition:
  """
  Attributes:
   - topicName: The name for the topic;

   - topicTalosResourceName: The Talos Resource Name for the topic;

   - partitionId: The PartitionId;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
    (2, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 2
    (3, TType.I32, 'partitionId', None, None, ), # 3
  )

  def __init__(self, topicName=None, topicTalosResourceName=None, partitionId=None,):
    self.topicName = topicName
    self.topicTalosResourceName = topicTalosResourceName
    self.partitionId = partitionId

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I32:
          self.partitionId = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TopicAndPartition')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 2)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.partitionId is not None:
      oprot.writeFieldBegin('partitionId', TType.I32, 3)
      oprot.writeI32(self.partitionId)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.partitionId is None:
      raise TProtocol.TProtocolException(message='Required field partitionId is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.partitionId)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class TopicInfo:
  """
  Attributes:
   - topicName: The name for the topic;

   - topicTalosResourceName: The Talos Resource Name for the topic;

   - ownerId: The ownerId for the toic, stored as 'developerId' which is invisible,
  but make authorization with 'accountId';

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
    (2, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 2
    (3, TType.STRING, 'ownerId', None, None, ), # 3
  )

  def __init__(self, topicName=None, topicTalosResourceName=None, ownerId=None,):
    self.topicName = topicName
    self.topicTalosResourceName = topicTalosResourceName
    self.ownerId = ownerId

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.ownerId = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TopicInfo')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 2)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.ownerId is not None:
      oprot.writeFieldBegin('ownerId', TType.STRING, 3)
      oprot.writeString(self.ownerId)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.ownerId is None:
      raise TProtocol.TProtocolException(message='Required field ownerId is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.ownerId)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class TopicAttribute:
  """
  Attributes:
   - partitionNumber: Partition number for the topic, default 8 (1 ~ 256)

   - messageRetentionSecs: The retention time(in secs) for message in the topic, talos will make sure
  that message in this topic will be available at least messageRetentionSecs,
  default 24h (1h ~ 365d)

   - attributes: Some other attribute;

   - oldPartitionNumber: Old partition number for the topic before reduced, default 0 (1 ~ 256)

  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'partitionNumber', None, None, ), # 1
    (2, TType.I32, 'messageRetentionSecs', None, None, ), # 2
    (3, TType.MAP, 'attributes', (TType.STRING,None,TType.STRING,None), None, ), # 3
    (4, TType.I32, 'oldPartitionNumber', None, None, ), # 4
  )

  def __init__(self, partitionNumber=None, messageRetentionSecs=None, attributes=None, oldPartitionNumber=None,):
    self.partitionNumber = partitionNumber
    self.messageRetentionSecs = messageRetentionSecs
    self.attributes = attributes
    self.oldPartitionNumber = oldPartitionNumber

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.partitionNumber = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.messageRetentionSecs = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.MAP:
          self.attributes = {}
          (_ktype1, _vtype2, _size0 ) = iprot.readMapBegin()
          for _i4 in range(_size0):
            _key5 = iprot.readString()
            _val6 = iprot.readString()
            self.attributes[_key5] = _val6
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I32:
          self.oldPartitionNumber = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TopicAttribute')
    if self.partitionNumber is not None:
      oprot.writeFieldBegin('partitionNumber', TType.I32, 1)
      oprot.writeI32(self.partitionNumber)
      oprot.writeFieldEnd()
    if self.messageRetentionSecs is not None:
      oprot.writeFieldBegin('messageRetentionSecs', TType.I32, 2)
      oprot.writeI32(self.messageRetentionSecs)
      oprot.writeFieldEnd()
    if self.attributes is not None:
      oprot.writeFieldBegin('attributes', TType.MAP, 3)
      oprot.writeMapBegin(TType.STRING, TType.STRING, len(self.attributes))
      for kiter7,viter8 in self.attributes.items():
        oprot.writeString(kiter7)
        oprot.writeString(viter8)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    if self.oldPartitionNumber is not None:
      oprot.writeFieldBegin('oldPartitionNumber', TType.I32, 4)
      oprot.writeI32(self.oldPartitionNumber)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.partitionNumber)
    value = (value * 31) ^ hash(self.messageRetentionSecs)
    value = (value * 31) ^ hash(self.attributes)
    value = (value * 31) ^ hash(self.oldPartitionNumber)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class PartitionState:
  """
  Attributes:
   - partitionId: Partition ID for this topic;

   - startOffset: The message start offset for this partition;

   - lastestOffset: The message lastest offset for this partition;

   - messageNumber: The message number in this partition;

   - messageBytes: The message bytes in this partition;

   - totalMessageNumber: The total message number of this partition
  optional for compatible forward

   - totalMessageBytes: The total message bytes of this partition
  optional for compatible forward

  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'partitionId', None, None, ), # 1
    (2, TType.I64, 'startOffset', None, None, ), # 2
    (3, TType.I64, 'lastestOffset', None, None, ), # 3
    (4, TType.I64, 'messageNumber', None, None, ), # 4
    (5, TType.I64, 'messageBytes', None, None, ), # 5
    (6, TType.I64, 'totalMessageNumber', None, None, ), # 6
    (7, TType.I64, 'totalMessageBytes', None, None, ), # 7
  )

  def __init__(self, partitionId=None, startOffset=None, lastestOffset=None, messageNumber=None, messageBytes=None, totalMessageNumber=None, totalMessageBytes=None,):
    self.partitionId = partitionId
    self.startOffset = startOffset
    self.lastestOffset = lastestOffset
    self.messageNumber = messageNumber
    self.messageBytes = messageBytes
    self.totalMessageNumber = totalMessageNumber
    self.totalMessageBytes = totalMessageBytes

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.partitionId = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.startOffset = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.lastestOffset = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I64:
          self.messageNumber = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.I64:
          self.messageBytes = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.I64:
          self.totalMessageNumber = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.I64:
          self.totalMessageBytes = iprot.readI64()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('PartitionState')
    if self.partitionId is not None:
      oprot.writeFieldBegin('partitionId', TType.I32, 1)
      oprot.writeI32(self.partitionId)
      oprot.writeFieldEnd()
    if self.startOffset is not None:
      oprot.writeFieldBegin('startOffset', TType.I64, 2)
      oprot.writeI64(self.startOffset)
      oprot.writeFieldEnd()
    if self.lastestOffset is not None:
      oprot.writeFieldBegin('lastestOffset', TType.I64, 3)
      oprot.writeI64(self.lastestOffset)
      oprot.writeFieldEnd()
    if self.messageNumber is not None:
      oprot.writeFieldBegin('messageNumber', TType.I64, 4)
      oprot.writeI64(self.messageNumber)
      oprot.writeFieldEnd()
    if self.messageBytes is not None:
      oprot.writeFieldBegin('messageBytes', TType.I64, 5)
      oprot.writeI64(self.messageBytes)
      oprot.writeFieldEnd()
    if self.totalMessageNumber is not None:
      oprot.writeFieldBegin('totalMessageNumber', TType.I64, 6)
      oprot.writeI64(self.totalMessageNumber)
      oprot.writeFieldEnd()
    if self.totalMessageBytes is not None:
      oprot.writeFieldBegin('totalMessageBytes', TType.I64, 7)
      oprot.writeI64(self.totalMessageBytes)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.partitionId is None:
      raise TProtocol.TProtocolException(message='Required field partitionId is unset!')
    if self.startOffset is None:
      raise TProtocol.TProtocolException(message='Required field startOffset is unset!')
    if self.lastestOffset is None:
      raise TProtocol.TProtocolException(message='Required field lastestOffset is unset!')
    if self.messageNumber is None:
      raise TProtocol.TProtocolException(message='Required field messageNumber is unset!')
    if self.messageBytes is None:
      raise TProtocol.TProtocolException(message='Required field messageBytes is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.partitionId)
    value = (value * 31) ^ hash(self.startOffset)
    value = (value * 31) ^ hash(self.lastestOffset)
    value = (value * 31) ^ hash(self.messageNumber)
    value = (value * 31) ^ hash(self.messageBytes)
    value = (value * 31) ^ hash(self.totalMessageNumber)
    value = (value * 31) ^ hash(self.totalMessageBytes)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class TopicState:
  """
  Attributes:
   - createTimestamp: Create Timestamp for this topic;

   - topicStatus: Topic status for this topic;

   - messageNumber: The message number in this topic(include all partitions);

   - messageBytes: The message bytes in this topic(include all partitions);

   - partitionStates: All partition state for this partition;

   - totalMessageNumber: The total message number of this topic,
  optional for compatible forward

   - totalMessageBytes: The total message bytes of this topic
  optional for compatible forward

  """

  thrift_spec = (
    None, # 0
    (1, TType.I64, 'createTimestamp', None, None, ), # 1
    (2, TType.I32, 'topicStatus', None, None, ), # 2
    (3, TType.I64, 'messageNumber', None, None, ), # 3
    (4, TType.I64, 'messageBytes', None, None, ), # 4
    (5, TType.LIST, 'partitionStates', (TType.STRUCT,(PartitionState, PartitionState.thrift_spec)), None, ), # 5
    (6, TType.I64, 'totalMessageNumber', None, None, ), # 6
    (7, TType.I64, 'totalMessageBytes', None, None, ), # 7
  )

  def __init__(self, createTimestamp=None, topicStatus=None, messageNumber=None, messageBytes=None, partitionStates=None, totalMessageNumber=None, totalMessageBytes=None,):
    self.createTimestamp = createTimestamp
    self.topicStatus = topicStatus
    self.messageNumber = messageNumber
    self.messageBytes = messageBytes
    self.partitionStates = partitionStates
    self.totalMessageNumber = totalMessageNumber
    self.totalMessageBytes = totalMessageBytes

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I64:
          self.createTimestamp = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.topicStatus = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.messageNumber = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I64:
          self.messageBytes = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.partitionStates = []
          (_etype12, _size9) = iprot.readListBegin()
          for _i13 in range(_size9):
            _elem14 = PartitionState()
            _elem14.read(iprot)
            self.partitionStates.append(_elem14)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.I64:
          self.totalMessageNumber = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.I64:
          self.totalMessageBytes = iprot.readI64()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TopicState')
    if self.createTimestamp is not None:
      oprot.writeFieldBegin('createTimestamp', TType.I64, 1)
      oprot.writeI64(self.createTimestamp)
      oprot.writeFieldEnd()
    if self.topicStatus is not None:
      oprot.writeFieldBegin('topicStatus', TType.I32, 2)
      oprot.writeI32(self.topicStatus)
      oprot.writeFieldEnd()
    if self.messageNumber is not None:
      oprot.writeFieldBegin('messageNumber', TType.I64, 3)
      oprot.writeI64(self.messageNumber)
      oprot.writeFieldEnd()
    if self.messageBytes is not None:
      oprot.writeFieldBegin('messageBytes', TType.I64, 4)
      oprot.writeI64(self.messageBytes)
      oprot.writeFieldEnd()
    if self.partitionStates is not None:
      oprot.writeFieldBegin('partitionStates', TType.LIST, 5)
      oprot.writeListBegin(TType.STRUCT, len(self.partitionStates))
      for iter15 in self.partitionStates:
        iter15.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.totalMessageNumber is not None:
      oprot.writeFieldBegin('totalMessageNumber', TType.I64, 6)
      oprot.writeI64(self.totalMessageNumber)
      oprot.writeFieldEnd()
    if self.totalMessageBytes is not None:
      oprot.writeFieldBegin('totalMessageBytes', TType.I64, 7)
      oprot.writeI64(self.totalMessageBytes)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.createTimestamp is None:
      raise TProtocol.TProtocolException(message='Required field createTimestamp is unset!')
    if self.topicStatus is None:
      raise TProtocol.TProtocolException(message='Required field topicStatus is unset!')
    if self.messageNumber is None:
      raise TProtocol.TProtocolException(message='Required field messageNumber is unset!')
    if self.messageBytes is None:
      raise TProtocol.TProtocolException(message='Required field messageBytes is unset!')
    if self.partitionStates is None:
      raise TProtocol.TProtocolException(message='Required field partitionStates is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.createTimestamp)
    value = (value * 31) ^ hash(self.topicStatus)
    value = (value * 31) ^ hash(self.messageNumber)
    value = (value * 31) ^ hash(self.messageBytes)
    value = (value * 31) ^ hash(self.partitionStates)
    value = (value * 31) ^ hash(self.totalMessageNumber)
    value = (value * 31) ^ hash(self.totalMessageBytes)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Topic:
  """
  Attributes:
   - topicInfo: TopicInfo for this topic;

   - topicAttribute: TopicAttribute for this topic;

   - topicState: TopicState for this topic;

   - topicQuota: TopicQuota for the topic;

   - topicAcl: TopicAcl for the topic;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicInfo', (TopicInfo, TopicInfo.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'topicAttribute', (TopicAttribute, TopicAttribute.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'topicState', (TopicState, TopicState.thrift_spec), None, ), # 3
    (4, TType.STRUCT, 'topicQuota', (talos.thrift.quota.ttypes.BaseQuota, talos.thrift.quota.ttypes.BaseQuota.thrift_spec), None, ), # 4
    (5, TType.MAP, 'topicAcl', (TType.STRING,None,TType.I32,None), None, ), # 5
  )

  def __init__(self, topicInfo=None, topicAttribute=None, topicState=None, topicQuota=None, topicAcl=None,):
    self.topicInfo = topicInfo
    self.topicAttribute = topicAttribute
    self.topicState = topicState
    self.topicQuota = topicQuota
    self.topicAcl = topicAcl

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicInfo = TopicInfo()
          self.topicInfo.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicAttribute = TopicAttribute()
          self.topicAttribute.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.topicState = TopicState()
          self.topicState.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRUCT:
          self.topicQuota = thrift.quota.ttypes.BaseQuota()
          self.topicQuota.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.MAP:
          self.topicAcl = {}
          (_ktype17, _vtype18, _size16 ) = iprot.readMapBegin()
          for _i20 in range(_size16):
            _key21 = iprot.readString()
            _val22 = iprot.readI32()
            self.topicAcl[_key21] = _val22
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Topic')
    if self.topicInfo is not None:
      oprot.writeFieldBegin('topicInfo', TType.STRUCT, 1)
      self.topicInfo.write(oprot)
      oprot.writeFieldEnd()
    if self.topicAttribute is not None:
      oprot.writeFieldBegin('topicAttribute', TType.STRUCT, 2)
      self.topicAttribute.write(oprot)
      oprot.writeFieldEnd()
    if self.topicState is not None:
      oprot.writeFieldBegin('topicState', TType.STRUCT, 3)
      self.topicState.write(oprot)
      oprot.writeFieldEnd()
    if self.topicQuota is not None:
      oprot.writeFieldBegin('topicQuota', TType.STRUCT, 4)
      self.topicQuota.write(oprot)
      oprot.writeFieldEnd()
    if self.topicAcl is not None:
      oprot.writeFieldBegin('topicAcl', TType.MAP, 5)
      oprot.writeMapBegin(TType.STRING, TType.I32, len(self.topicAcl))
      for kiter23,viter24 in self.topicAcl.items():
        oprot.writeString(kiter23)
        oprot.writeI32(viter24)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicInfo is None:
      raise TProtocol.TProtocolException(message='Required field topicInfo is unset!')
    if self.topicAttribute is None:
      raise TProtocol.TProtocolException(message='Required field topicAttribute is unset!')
    if self.topicState is None:
      raise TProtocol.TProtocolException(message='Required field topicState is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicInfo)
    value = (value * 31) ^ hash(self.topicAttribute)
    value = (value * 31) ^ hash(self.topicState)
    value = (value * 31) ^ hash(self.topicQuota)
    value = (value * 31) ^ hash(self.topicAcl)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class CreateTopicRequest:
  """
  Attributes:
   - topicName: Topic name;

   - topicAttribute: TopicAttribute for the topic, partitionNumber is required;

   - topicQuota: TopicQuota for the topic

   - aclMap: TopicAcl for the topic
  Note: Grantee must specified 'GrantType' and 'identifier', specifically,
  the 'identifier' must be the 'appId'/'accountId' in openplatform of Xiaomi

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
    (2, TType.STRUCT, 'topicAttribute', (TopicAttribute, TopicAttribute.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'topicQuota', (talos.thrift.quota.ttypes.BaseQuota, talos.thrift.quota.ttypes.BaseQuota.thrift_spec), None, ), # 3
    (4, TType.MAP, 'aclMap', (TType.STRUCT,(talos.thrift.authorization.ttypes.Grantee, talos.thrift.authorization.ttypes.Grantee.thrift_spec),TType.I32,None), None, ), # 4
  )

  def __init__(self, topicName=None, topicAttribute=None, topicQuota=None, aclMap=None,):
    self.topicName = topicName
    self.topicAttribute = topicAttribute
    self.topicQuota = topicQuota
    self.aclMap = aclMap

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicAttribute = TopicAttribute()
          self.topicAttribute.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.topicQuota = talos.thrift.quota.ttypes.BaseQuota()
          self.topicQuota.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.MAP:
          self.aclMap = {}
          (_ktype26, _vtype27, _size25 ) = iprot.readMapBegin()
          for _i29 in range(_size25):
            _key30 = talos.thrift.authorization.ttypes.Grantee()
            _key30.read(iprot)
            _val31 = iprot.readI32()
            self.aclMap[_key30] = _val31
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('CreateTopicRequest')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    if self.topicAttribute is not None:
      oprot.writeFieldBegin('topicAttribute', TType.STRUCT, 2)
      self.topicAttribute.write(oprot)
      oprot.writeFieldEnd()
    if self.topicQuota is not None:
      oprot.writeFieldBegin('topicQuota', TType.STRUCT, 3)
      self.topicQuota.write(oprot)
      oprot.writeFieldEnd()
    if self.aclMap is not None:
      oprot.writeFieldBegin('aclMap', TType.MAP, 4)
      oprot.writeMapBegin(TType.STRUCT, TType.I32, len(self.aclMap))
      for kiter32,viter33 in self.aclMap.items():
        kiter32.write(oprot)
        oprot.writeI32(viter33)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    if self.topicAttribute is None:
      raise TProtocol.TProtocolException(message='Required field topicAttribute is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    value = (value * 31) ^ hash(self.topicAttribute)
    value = (value * 31) ^ hash(self.topicQuota)
    value = (value * 31) ^ hash(self.aclMap)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class CreateReplicationTopicRequest:
  """
  Attributes:
   - topicName: Topic name;

   - sourceClusterList: source cluster list;

   - sourceTopicName: source topic name

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
    (2, TType.STRING, 'sourceClusterList', None, None, ), # 2
    (3, TType.STRING, 'sourceTopicName', None, None, ), # 3
  )

  def __init__(self, topicName=None, sourceClusterList=None, sourceTopicName=None,):
    self.topicName = topicName
    self.sourceClusterList = sourceClusterList
    self.sourceTopicName = sourceTopicName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.sourceClusterList = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.sourceTopicName = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('CreateReplicationTopicRequest')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    if self.sourceClusterList is not None:
      oprot.writeFieldBegin('sourceClusterList', TType.STRING, 2)
      oprot.writeString(self.sourceClusterList)
      oprot.writeFieldEnd()
    if self.sourceTopicName is not None:
      oprot.writeFieldBegin('sourceTopicName', TType.STRING, 3)
      oprot.writeString(self.sourceTopicName)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    if self.sourceClusterList is None:
      raise TProtocol.TProtocolException(message='Required field sourceClusterList is unset!')
    if self.sourceTopicName is None:
      raise TProtocol.TProtocolException(message='Required field sourceTopicName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    value = (value * 31) ^ hash(self.sourceClusterList)
    value = (value * 31) ^ hash(self.sourceTopicName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class CreateTopicResponse:
  """
  Attributes:
   - topicInfo: TopicInfo for the created topic;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicInfo', (TopicInfo, TopicInfo.thrift_spec), None, ), # 1
  )

  def __init__(self, topicInfo=None,):
    self.topicInfo = topicInfo

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicInfo = TopicInfo()
          self.topicInfo.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('CreateTopicResponse')
    if self.topicInfo is not None:
      oprot.writeFieldBegin('topicInfo', TType.STRUCT, 1)
      self.topicInfo.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicInfo is None:
      raise TProtocol.TProtocolException(message='Required field topicInfo is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicInfo)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class DeleteTopicRequest:
  """
  Attributes:
   - topicTalosResourceName: The topicTalosResourceName is in TopicInfo that returned by CreateTopic,
  you can also get TopicInfo by describeTopic;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
  )

  def __init__(self, topicTalosResourceName=None,):
    self.topicTalosResourceName = topicTalosResourceName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('DeleteTopicRequest')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 1)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ChangeTopicAttributeRequest:
  """
  Attributes:
   - topicTalosResourceName: The topicTalosResourceName is in TopicInfo that returned by CreateTopic,
  you can also get TopicInfo by describeTopic;

   - topicAttribute: Topic attribute;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'topicAttribute', (TopicAttribute, TopicAttribute.thrift_spec), None, ), # 2
  )

  def __init__(self, topicTalosResourceName=None, topicAttribute=None,):
    self.topicTalosResourceName = topicTalosResourceName
    self.topicAttribute = topicAttribute

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicAttribute = TopicAttribute()
          self.topicAttribute.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ChangeTopicAttributeRequest')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 1)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.topicAttribute is not None:
      oprot.writeFieldBegin('topicAttribute', TType.STRUCT, 2)
      self.topicAttribute.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.topicAttribute is None:
      raise TProtocol.TProtocolException(message='Required field topicAttribute is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.topicAttribute)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class DescribeTopicRequest:
  """
  Attributes:
   - topicName: Topic name;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
  )

  def __init__(self, topicName=None,):
    self.topicName = topicName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('DescribeTopicRequest')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetDescribeInfoRequest:
  """
  Attributes:
   - topicName: Topic name;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
  )

  def __init__(self, topicName=None,):
    self.topicName = topicName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetDescribeInfoRequest')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetTopicAttributeRequest:
  """
  Attributes:
   - topicName: Topic name;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'topicName', None, None, ), # 1
  )

  def __init__(self, topicName=None,):
    self.topicName = topicName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.topicName = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetTopicAttributeRequest')
    if self.topicName is not None:
      oprot.writeFieldBegin('topicName', TType.STRING, 1)
      oprot.writeString(self.topicName)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicName is None:
      raise TProtocol.TProtocolException(message='Required field topicName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class AddSubResourceNameRequest:
  """
  Attributes:
   - topicTalosResourceName: Add subResourceName for specified resourceName, after 'addSubResourceName',
  one can visit topic by a Role of Cloud-Manager with specified teamId

   - orgId: OrgId which the topic transfer to, must starts with 'CL'

   - adminTeamId: TeamId which specified the admin of the topic, must starts with 'CI'

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRING, 'orgId', None, None, ), # 2
    (3, TType.STRING, 'adminTeamId', None, None, ), # 3
  )

  def __init__(self, topicTalosResourceName=None, orgId=None, adminTeamId=None,):
    self.topicTalosResourceName = topicTalosResourceName
    self.orgId = orgId
    self.adminTeamId = adminTeamId

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.orgId = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.adminTeamId = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('AddSubResourceNameRequest')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 1)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.orgId is not None:
      oprot.writeFieldBegin('orgId', TType.STRING, 2)
      oprot.writeString(self.orgId)
      oprot.writeFieldEnd()
    if self.adminTeamId is not None:
      oprot.writeFieldBegin('adminTeamId', TType.STRING, 3)
      oprot.writeString(self.adminTeamId)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.orgId is None:
      raise TProtocol.TProtocolException(message='Required field orgId is unset!')
    if self.adminTeamId is None:
      raise TProtocol.TProtocolException(message='Required field adminTeamId is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.orgId)
    value = (value * 31) ^ hash(self.adminTeamId)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class DescribeTopicResponse:
  """
  Attributes:
   - topicInfo: Topic info;

   - topicAttribute: Topic attribute;

   - topicState: Topic state;

   - topicQuota: Topic quota;

   - aclMap: Topic Acl

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicInfo', (TopicInfo, TopicInfo.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'topicAttribute', (TopicAttribute, TopicAttribute.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'topicState', (TopicState, TopicState.thrift_spec), None, ), # 3
    (4, TType.STRUCT, 'topicQuota', (talos.thrift.quota.ttypes.BaseQuota, talos.thrift.quota.ttypes.BaseQuota.thrift_spec), None, ), # 4
    (5, TType.MAP, 'aclMap', (TType.STRING,None,TType.I32,None), None, ), # 5
  )

  def __init__(self, topicInfo=None, topicAttribute=None, topicState=None, topicQuota=None, aclMap=None,):
    self.topicInfo = topicInfo
    self.topicAttribute = topicAttribute
    self.topicState = topicState
    self.topicQuota = topicQuota
    self.aclMap = aclMap

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicInfo = TopicInfo()
          self.topicInfo.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicAttribute = TopicAttribute()
          self.topicAttribute.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.topicState = TopicState()
          self.topicState.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRUCT:
          self.topicQuota = talos.thrift.quota.ttypes.BaseQuota()
          self.topicQuota.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.MAP:
          self.aclMap = {}
          (_ktype35, _vtype36, _size34 ) = iprot.readMapBegin()
          for _i38 in range(_size34):
            _key39 = iprot.readString()
            _val40 = iprot.readI32()
            self.aclMap[_key39] = _val40
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('DescribeTopicResponse')
    if self.topicInfo is not None:
      oprot.writeFieldBegin('topicInfo', TType.STRUCT, 1)
      self.topicInfo.write(oprot)
      oprot.writeFieldEnd()
    if self.topicAttribute is not None:
      oprot.writeFieldBegin('topicAttribute', TType.STRUCT, 2)
      self.topicAttribute.write(oprot)
      oprot.writeFieldEnd()
    if self.topicState is not None:
      oprot.writeFieldBegin('topicState', TType.STRUCT, 3)
      self.topicState.write(oprot)
      oprot.writeFieldEnd()
    if self.topicQuota is not None:
      oprot.writeFieldBegin('topicQuota', TType.STRUCT, 4)
      self.topicQuota.write(oprot)
      oprot.writeFieldEnd()
    if self.aclMap is not None:
      oprot.writeFieldBegin('aclMap', TType.MAP, 5)
      oprot.writeMapBegin(TType.STRING, TType.I32, len(self.aclMap))
      for kiter41,viter42 in self.aclMap.items():
        oprot.writeString(kiter41)
        oprot.writeI32(viter42)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicInfo is None:
      raise TProtocol.TProtocolException(message='Required field topicInfo is unset!')
    if self.topicAttribute is None:
      raise TProtocol.TProtocolException(message='Required field topicAttribute is unset!')
    if self.topicState is None:
      raise TProtocol.TProtocolException(message='Required field topicState is unset!')
    if self.topicQuota is None:
      raise TProtocol.TProtocolException(message='Required field topicQuota is unset!')
    if self.aclMap is None:
      raise TProtocol.TProtocolException(message='Required field aclMap is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicInfo)
    value = (value * 31) ^ hash(self.topicAttribute)
    value = (value * 31) ^ hash(self.topicState)
    value = (value * 31) ^ hash(self.topicQuota)
    value = (value * 31) ^ hash(self.aclMap)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetDescribeInfoResponse:
  """
  Attributes:
   - topicTalosResourceName: Topic resourceName;

   - partitionNumber: Topic partitionNumber;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.I32, 'partitionNumber', None, None, ), # 2
  )

  def __init__(self, topicTalosResourceName=None, partitionNumber=None,):
    self.topicTalosResourceName = topicTalosResourceName
    self.partitionNumber = partitionNumber

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.partitionNumber = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetDescribeInfoResponse')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 1)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.partitionNumber is not None:
      oprot.writeFieldBegin('partitionNumber', TType.I32, 2)
      oprot.writeI32(self.partitionNumber)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.partitionNumber is None:
      raise TProtocol.TProtocolException(message='Required field partitionNumber is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.partitionNumber)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetTopicAttributeResponse:
  """
  Attributes:
   - topicTalosResourceName: Topic resourceName;

   - topicAttribute: Topic Attribute;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicTalosResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'topicAttribute', (TopicAttribute, TopicAttribute.thrift_spec), None, ), # 2
  )

  def __init__(self, topicTalosResourceName=None, topicAttribute=None,):
    self.topicTalosResourceName = topicTalosResourceName
    self.topicAttribute = topicAttribute

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicTalosResourceName = TopicTalosResourceName()
          self.topicTalosResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicAttribute = TopicAttribute()
          self.topicAttribute.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetTopicAttributeResponse')
    if self.topicTalosResourceName is not None:
      oprot.writeFieldBegin('topicTalosResourceName', TType.STRUCT, 1)
      self.topicTalosResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.topicAttribute is not None:
      oprot.writeFieldBegin('topicAttribute', TType.STRUCT, 2)
      self.topicAttribute.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicTalosResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicTalosResourceName is unset!')
    if self.topicAttribute is None:
      raise TProtocol.TProtocolException(message='Required field topicAttribute is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicTalosResourceName)
    value = (value * 31) ^ hash(self.topicAttribute)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ListTopicsResponse:
  """
  Attributes:
   - topicInfos: The topicInfos;

  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'topicInfos', (TType.STRUCT,(TopicInfo, TopicInfo.thrift_spec)), None, ), # 1
  )

  def __init__(self, topicInfos=None,):
    self.topicInfos = topicInfos

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.topicInfos = []
          (_etype46, _size43) = iprot.readListBegin()
          for _i47 in range(_size43):
            _elem48 = TopicInfo()
            _elem48.read(iprot)
            self.topicInfos.append(_elem48)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ListTopicsResponse')
    if self.topicInfos is not None:
      oprot.writeFieldBegin('topicInfos', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.topicInfos))
      for iter49 in self.topicInfos:
        iter49.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicInfos is None:
      raise TProtocol.TProtocolException(message='Required field topicInfos is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicInfos)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ListTopicsInfoResponse:
  """
  Attributes:
   - topicList: The topics' information;

  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'topicList', (TType.STRUCT,(Topic, Topic.thrift_spec)), None, ), # 1
  )

  def __init__(self, topicList=None,):
    self.topicList = topicList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.topicList = []
          (_etype53, _size50) = iprot.readListBegin()
          for _i54 in range(_size50):
            _elem55 = Topic()
            _elem55.read(iprot)
            self.topicList.append(_elem55)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ListTopicsInfoResponse')
    if self.topicList is not None:
      oprot.writeFieldBegin('topicList', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.topicList))
      for iter56 in self.topicList:
        iter56.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicList is None:
      raise TProtocol.TProtocolException(message='Required field topicList is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class SetTopicQuotaRequest:
  """
  Attributes:
   - topicResourceName: The topic resource name

   - topicQuota: The topic quota

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'topicQuota', (talos.thrift.quota.ttypes.BaseQuota, talos.thrift.quota.ttypes.BaseQuota.thrift_spec), None, ), # 2
  )

  def __init__(self, topicResourceName=None, topicQuota=None,):
    self.topicResourceName = topicResourceName
    self.topicQuota = topicQuota

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.topicQuota = talos.thrift.quota.ttypes.BaseQuota()
          self.topicQuota.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('SetTopicQuotaRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.topicQuota is not None:
      oprot.writeFieldBegin('topicQuota', TType.STRUCT, 2)
      self.topicQuota.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    if self.topicQuota is None:
      raise TProtocol.TProtocolException(message='Required field topicQuota is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    value = (value * 31) ^ hash(self.topicQuota)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class QueryTopicQuotaRequest:
  """
  Attributes:
   - topicResourceName: The topic resource name

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
  )

  def __init__(self, topicResourceName=None,):
    self.topicResourceName = topicResourceName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('QueryTopicQuotaRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class QueryTopicQuotaResponse:
  """
  Attributes:
   - topicQuota: The topic quota

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicQuota', (talos.thrift.quota.ttypes.BaseQuota, talos.thrift.quota.ttypes.BaseQuota.thrift_spec), None, ), # 1
  )

  def __init__(self, topicQuota=None,):
    self.topicQuota = topicQuota

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicQuota = talos.thrift.quota.ttypes.BaseQuota()
          self.topicQuota.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('QueryTopicQuotaResponse')
    if self.topicQuota is not None:
      oprot.writeFieldBegin('topicQuota', TType.STRUCT, 1)
      self.topicQuota.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicQuota is None:
      raise TProtocol.TProtocolException(message='Required field topicQuota is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicQuota)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class DeleteTopicQuotaRequest:
  """
  Attributes:
   - topicResourceName: The topic resource name

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
  )

  def __init__(self, topicResourceName=None,):
    self.topicResourceName = topicResourceName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('DeleteTopicQuotaRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class SetPermissionRequest:
  """
  Attributes:
   - topicResourceName: Topic resource name for which to set permission;

   - grant: The Grantee to set permission;
  the Grantee must be specified a appId or accountId.
  we support for cross-authorization based on 'accountId'

   - permission: The permission to set;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'grant', (talos.thrift.authorization.ttypes.Grantee, talos.thrift.authorization.ttypes.Grantee.thrift_spec), None, ), # 2
    (3, TType.I32, 'permission', None, None, ), # 3
  )

  def __init__(self, topicResourceName=None, grant=None, permission=None,):
    self.topicResourceName = topicResourceName
    self.grant = grant
    self.permission = permission

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.grant = talos.thrift.authorization.ttypes.Grantee()
          self.grant.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I32:
          self.permission = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('SetPermissionRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.grant is not None:
      oprot.writeFieldBegin('grant', TType.STRUCT, 2)
      self.grant.write(oprot)
      oprot.writeFieldEnd()
    if self.permission is not None:
      oprot.writeFieldBegin('permission', TType.I32, 3)
      oprot.writeI32(self.permission)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    if self.grant is None:
      raise TProtocol.TProtocolException(message='Required field grant is unset!')
    if self.permission is None:
      raise TProtocol.TProtocolException(message='Required field permission is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    value = (value * 31) ^ hash(self.grant)
    value = (value * 31) ^ hash(self.permission)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class RevokePermissionRequest:
  """
  Attributes:
   - topicResourceName: Topic resource name for which to revoke permission;

   - grant: The Grantee to revoke permission;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'grant', (talos.thrift.authorization.ttypes.Grantee, talos.thrift.authorization.ttypes.Grantee.thrift_spec), None, ), # 2
  )

  def __init__(self, topicResourceName=None, grant=None,):
    self.topicResourceName = topicResourceName
    self.grant = grant

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.grant = talos.thrift.authorization.ttypes.Grantee()
          self.grant.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('RevokePermissionRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.grant is not None:
      oprot.writeFieldBegin('grant', TType.STRUCT, 2)
      self.grant.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    if self.grant is None:
      raise TProtocol.TProtocolException(message='Required field grant is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    value = (value * 31) ^ hash(self.grant)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ListPermissionRequest:
  """
  Attributes:
   - topicResourceName: Topic resource name for which to list permission;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
  )

  def __init__(self, topicResourceName=None,):
    self.topicResourceName = topicResourceName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ListPermissionRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ListPermissionResponse:
  """
  Attributes:
   - permissions: The principal string to permission value map;

  """

  thrift_spec = (
    None, # 0
    (1, TType.MAP, 'permissions', (TType.STRING,None,TType.I32,None), None, ), # 1
  )

  def __init__(self, permissions=None,):
    self.permissions = permissions

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.MAP:
          self.permissions = {}
          (_ktype58, _vtype59, _size57 ) = iprot.readMapBegin()
          for _i61 in range(_size57):
            _key62 = iprot.readString()
            _val63 = iprot.readI32()
            self.permissions[_key62] = _val63
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ListPermissionResponse')
    if self.permissions is not None:
      oprot.writeFieldBegin('permissions', TType.MAP, 1)
      oprot.writeMapBegin(TType.STRING, TType.I32, len(self.permissions))
      for kiter64,viter65 in self.permissions.items():
        oprot.writeString(kiter64)
        oprot.writeI32(viter65)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.permissions is None:
      raise TProtocol.TProtocolException(message='Required field permissions is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.permissions)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetPermissionRequest:
  """
  Attributes:
   - topicResourceName: Topic resource name for which to get permission;

   - grant: The Grantee to get permission;

  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'topicResourceName', (TopicTalosResourceName, TopicTalosResourceName.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'grant', (talos.thrift.authorization.ttypes.Grantee, talos.thrift.authorization.ttypes.Grantee.thrift_spec), None, ), # 2
  )

  def __init__(self, topicResourceName=None, grant=None,):
    self.topicResourceName = topicResourceName
    self.grant = grant

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.topicResourceName = TopicTalosResourceName()
          self.topicResourceName.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.grant = talos.thrift.authorization.ttypes.Grantee()
          self.grant.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetPermissionRequest')
    if self.topicResourceName is not None:
      oprot.writeFieldBegin('topicResourceName', TType.STRUCT, 1)
      self.topicResourceName.write(oprot)
      oprot.writeFieldEnd()
    if self.grant is not None:
      oprot.writeFieldBegin('grant', TType.STRUCT, 2)
      self.grant.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.topicResourceName is None:
      raise TProtocol.TProtocolException(message='Required field topicResourceName is unset!')
    if self.grant is None:
      raise TProtocol.TProtocolException(message='Required field grant is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.topicResourceName)
    value = (value * 31) ^ hash(self.grant)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class GetPermissionResponse:
  """
  Attributes:
   - permission: The permission;

  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'permission', None, None, ), # 1
  )

  def __init__(self, permission=None,):
    self.permission = permission

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.permission = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('GetPermissionResponse')
    if self.permission is not None:
      oprot.writeFieldBegin('permission', TType.I32, 1)
      oprot.writeI32(self.permission)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.permission is None:
      raise TProtocol.TProtocolException(message='Required field permission is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.permission)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#coding=utf-8
#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from talos.thrift.Thrift import TType, TMessageType, TException, TApplicationException
import talos.thrift.common.ttypes


from talos.thrift.transport import TTransport
from talos.thrift.protocol import TBinaryProtocol, TProtocol
try:
  from talos.thrift.protocol import fastbinary
except:
  fastbinary = None


class UserType:
  """
  小米存储系统认证信息类型
  """
  DEV_XIAOMI_SSO = 1
  DEV_XIAOMI = 2
  APP_SECRET = 10
  APP_ACCESS_TOKEN = 11
  APP_XIAOMI_SSO = 12
  APP_ANONYMOUS = 13

  _VALUES_TO_NAMES = {
    1: "DEV_XIAOMI_SSO",
    2: "DEV_XIAOMI",
    10: "APP_SECRET",
    11: "APP_ACCESS_TOKEN",
    12: "APP_XIAOMI_SSO",
    13: "APP_ANONYMOUS",
  }

  _NAMES_TO_VALUES = {
    "DEV_XIAOMI_SSO": 1,
    "DEV_XIAOMI": 2,
    "APP_SECRET": 10,
    "APP_ACCESS_TOKEN": 11,
    "APP_XIAOMI_SSO": 12,
    "APP_ANONYMOUS": 13,
  }

class MacAlgorithm:
  """
  签名使用的HMMAC算法
  """
  HmacMD5 = 1
  HmacSHA1 = 2
  HmacSHA256 = 3

  _VALUES_TO_NAMES = {
    1: "HmacMD5",
    2: "HmacSHA1",
    3: "HmacSHA256",
  }

  _NAMES_TO_VALUES = {
    "HmacMD5": 1,
    "HmacSHA1": 2,
    "HmacSHA256": 3,
  }

class AppUserAuthProvider:
  """
  第三方身份认证提供方，用于认证应用用户(非开发者)。
  目前提供小米SSO和几种常见OAuth系统
  """
  XIAOMI_SSO = 1
  XIAOMI_OAUTH = 2
  QQ_OAUTH = 3
  SINA_OAUTH = 4
  RENREN_OAUTH = 5

  _VALUES_TO_NAMES = {
    1: "XIAOMI_SSO",
    2: "XIAOMI_OAUTH",
    3: "QQ_OAUTH",
    4: "SINA_OAUTH",
    5: "RENREN_OAUTH",
  }

  _NAMES_TO_VALUES = {
    "XIAOMI_SSO": 1,
    "XIAOMI_OAUTH": 2,
    "QQ_OAUTH": 3,
    "SINA_OAUTH": 4,
    "RENREN_OAUTH": 5,
  }


class Credential:
  """
  小米存储系统认证信息

  Attributes:
   - type: 用户登录类型
   - secretKeyId: 用于服务端查询SecretKey的键值:
  1) userId: 对应User Secret
  2) appId: 对应App Secret，匿名登录也需设置
  3) storageAccessTokenId: 对应Storage Access Token
   - secretKey: Secret Key
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'type', None, None, ), # 1
    (2, TType.STRING, 'secretKeyId', None, None, ), # 2
    (3, TType.STRING, 'secretKey', None, None, ), # 3
  )

  def __init__(self, type=None, secretKeyId=None, secretKey=None):
    self.type = type
    self.secretKeyId = secretKeyId
    self.secretKey = secretKey

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.type = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.secretKeyId = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.secretKey = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Credential')
    if self.type is not None:
      oprot.writeFieldBegin('type', TType.I32, 1)
      oprot.writeI32(self.type)
      oprot.writeFieldEnd()
    if self.secretKeyId is not None:
      oprot.writeFieldBegin('secretKeyId', TType.STRING, 2)
      oprot.writeString(self.secretKeyId)
      oprot.writeFieldEnd()
    if self.secretKey is not None:
      oprot.writeFieldBegin('secretKey', TType.STRING, 3)
      oprot.writeString(self.secretKey)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.type)
    value = (value * 31) ^ hash(self.secretKeyId)
    value = (value * 31) ^ hash(self.secretKey)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class HttpAuthorizationHeader:
  """
  Authorization头包含的内容

  Attributes:
   - version
   - userType
   - secretKeyId
   - secretKey: 直接使用sercetKey，此项被设置时，signature将被忽略，
  非安全传输应使用签名
   - signature: 如secretKey未设置，则认为使用签名，此时必须设置，
  被签名的正文格式：header1[\nheader2[\nheader3[...]]]，
  如使用默认SUGGESTED_SIGNATURE_HEADERS时为：$host\n$timestamp\n$md5
   - algorithm: 签名HMAC算法，客户端可定制，
  使用签名时必须设置
   - signedHeaders: 包含所有签名涉及到的部分，建议使用SUGGESTED_SIGNATURE_HEADERS，
  服务端未强制必须使用所列headers，定制的client自己负责签名的安全强度，
  使用签名时必须设置
   - supportAccountKey
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'version', None, "SDS-V1", ), # 1
    (2, TType.I32, 'userType', None,     13, ), # 2
    (3, TType.STRING, 'secretKeyId', None, None, ), # 3
    (4, TType.STRING, 'secretKey', None, None, ), # 4
    (5, TType.STRING, 'signature', None, None, ), # 5
    (6, TType.I32, 'algorithm', None, None, ), # 6
    (7, TType.LIST, 'signedHeaders', (TType.STRING,None), [
    ], ), # 7
    (8, TType.BOOL, 'supportAccountKey', None, False, ), # 8
  )

  def __init__(self, version=thrift_spec[1][4], userType=thrift_spec[2][4], secretKeyId=None, secretKey=None, signature=None, algorithm=None, signedHeaders=thrift_spec[7][4], supportAccountKey=thrift_spec[8][4],):
    self.version = version
    self.userType = userType
    self.secretKeyId = secretKeyId
    self.secretKey = secretKey
    self.signature = signature
    self.algorithm = algorithm
    if signedHeaders is self.thrift_spec[7][4]:
      signedHeaders = [
    ]
    self.signedHeaders = signedHeaders
    self.supportAccountKey = supportAccountKey

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.version = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.userType = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.secretKeyId = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.secretKey = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.signature = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.I32:
          self.algorithm = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.LIST:
          self.signedHeaders = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in range(_size0):
            _elem5 = iprot.readString()
            self.signedHeaders.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.BOOL:
          self.supportAccountKey = iprot.readBool()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('HttpAuthorizationHeader')
    if self.version is not None:
      oprot.writeFieldBegin('version', TType.STRING, 1)
      oprot.writeString(self.version)
      oprot.writeFieldEnd()
    if self.userType is not None:
      oprot.writeFieldBegin('userType', TType.I32, 2)
      oprot.writeI32(self.userType)
      oprot.writeFieldEnd()
    if self.secretKeyId is not None:
      oprot.writeFieldBegin('secretKeyId', TType.STRING, 3)
      oprot.writeString(self.secretKeyId)
      oprot.writeFieldEnd()
    if self.secretKey is not None:
      oprot.writeFieldBegin('secretKey', TType.STRING, 4)
      oprot.writeString(self.secretKey)
      oprot.writeFieldEnd()
    if self.signature is not None:
      oprot.writeFieldBegin('signature', TType.STRING, 5)
      oprot.writeString(self.signature)
      oprot.writeFieldEnd()
    if self.algorithm is not None:
      oprot.writeFieldBegin('algorithm', TType.I32, 6)
      oprot.writeI32(self.algorithm)
      oprot.writeFieldEnd()
    if self.signedHeaders is not None:
      oprot.writeFieldBegin('signedHeaders', TType.LIST, 7)
      oprot.writeListBegin(TType.STRING, len(self.signedHeaders))
      for iter6 in self.signedHeaders:
        oprot.writeString(iter6)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.supportAccountKey is not None:
      oprot.writeFieldBegin('supportAccountKey', TType.BOOL, 8)
      oprot.writeBool(self.supportAccountKey)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.version)
    value = (value * 31) ^ hash(self.userType)
    value = (value * 31) ^ hash(self.secretKeyId)
    value = (value * 31) ^ hash(self.secretKey)
    value = (value * 31) ^ hash(self.signature)
    value = (value * 31) ^ hash(self.algorithm)
    value = (value * 31) ^ hash(self.signedHeaders)
    value = (value * 31) ^ hash(self.supportAccountKey)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)
